package xyz.wbsite.optimizer;

import com.googlecode.htmlcompressor.compressor.HtmlCompressor;
import com.googlecode.htmlcompressor.compressor.XmlCompressor;
import com.googlecode.htmlcompressor.compressor.YuiCssCompressor;
import com.googlecode.htmlcompressor.compressor.YuiJavaScriptCompressor;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.codehaus.plexus.util.FileUtils;
import xyz.wbsite.optimizer.compressor.SimpleJavaScriptCompressor;
import xyz.wbsite.optimizer.task.Task;
import xyz.wbsite.optimizer.util.FileUtil;

import java.io.File;
import java.io.IOException;
import java.util.List;

/**
 * @goal run
 * @phase compile
 * @execute phase="compile"
 */
public class Optimizer extends AbstractMojo {

    /**
     * @parameter
     */
    private Task[] tasks;

    public void execute() throws MojoExecutionException {
        getLog().info("------------------------------------------------------------------------");
        for (Task task : tasks) {
            try {
                File directory = task.getDirectory();
                File targetPath = task.getTargetPath();
                String include = task.getInclude();
                String exclude = task.getExclude();

                List fileList = FileUtils.getFiles(directory, include, exclude);
                for (Object o : fileList) {
                    if (o instanceof File) {
                        File sourceFile = (File) o;
                        File targetFile = new File(targetPath, sourceFile.getAbsolutePath().substring(directory.getAbsolutePath().length()));
                        switch (task.getTaskType()) {
                            case COMPRESS_PLAIN:
                                compressPLAIN(sourceFile, targetFile);
                                break;
                            case COMPRESS_CSS:
                                compressCSS(sourceFile, targetFile);
                                break;
                            case COMPRESS_JS:
                                compressJS(sourceFile, targetFile);
                                break;
                            case COMPRESS_HTML:
                                compressHTML(sourceFile, targetFile);
                                break;
                            case COMPRESS_XML:
                                compressXML(sourceFile, targetFile);
                                break;
                        }
                    }
                }
            } catch (IOException e) {
//                e.printStackTrace();
            }
        }
        getLog().info("------------------------------------------------------------------------");
        getLog().info("The optimization task has been completed.");
    }

    public void compressPLAIN(File sourceFile, File targetFile) {
        try {
            // 源文件大小
            long sourceLength = sourceFile.length();
            // 从源文件向目标复制文件
            FileUtils.copyFile(sourceFile, targetFile);
            // 对目标文件进行压缩
            String string = FileUtil.readFileToString(sourceFile);
            String targetString = FileUtil.trim(string);
            FileUtil.writeStringToFile(targetString, targetFile, false, false);
            // 目标文件压缩后的大小
            long targetLength = targetFile.length();
            // 打印压缩信息
            getLog().info(targetFile.getName() + FileUtil.compressorInfo(sourceLength, targetLength));
        } catch (Exception e) {
            e.printStackTrace();
            getLog().info(sourceFile.getAbsolutePath() + " compress failed");
        }
    }

    public void compressCSS(File sourceFile, File targetFile) {
        try {
            // 源文件大小
            long sourceLength = sourceFile.length();
            // 从源文件向目标复制文件
            FileUtils.copyFile(sourceFile, targetFile);
            // 对目标文件进行压缩
            YuiCssCompressor compressor = new YuiCssCompressor();
            String compress = compressor.compress(FileUtil.readFileToString(sourceFile));
            FileUtil.writeStringToFile(compress, targetFile, false, false);
            // 目标文件压缩后的大小
            long targetLength = targetFile.length();
            // 打印压缩信息
            getLog().info(targetFile.getName() + FileUtil.compressorInfo(sourceLength, targetLength));
        } catch (Exception e) {
            e.printStackTrace();
            getLog().info(sourceFile.getAbsolutePath() + " compress failed");
        }
    }

    public void compressJS(File sourceFile, File targetFile) {
        try {
            // 源文件大小
            long sourceLength = sourceFile.length();
            // 从源文件向目标复制文件
            FileUtils.copyFile(sourceFile, targetFile);
            // 对目标文件进行压缩
            YuiJavaScriptCompressor compressor = new YuiJavaScriptCompressor();
            String compress = compressor.compress(FileUtil.readFileToString(sourceFile));
            FileUtil.writeStringToFile(compress, targetFile, false, false);
            // 目标文件压缩后的大小
            long targetLength = targetFile.length();
            // 打印压缩信息
            getLog().info(targetFile.getName() + FileUtil.compressorInfo(sourceLength, targetLength));
        } catch (Exception e) {
            e.printStackTrace();
            getLog().info(sourceFile.getAbsolutePath() + " compress failed");
        }
    }

    public void compressHTML(File sourceFile, File targetFile) {
        try {
            // 源文件大小
            long sourceLength = sourceFile.length();
            // 从源文件向目标复制文件
            FileUtils.copyFile(sourceFile, targetFile);
            // 对目标文件进行压缩
            HtmlCompressor compressor = new HtmlCompressor();
            compressor.setRemoveIntertagSpaces(true);
            compressor.setCompressJavaScript(true);
            compressor.setJavaScriptCompressor(new SimpleJavaScriptCompressor());
            compressor.setCompressCss(true);
            String compress = compressor.compress(FileUtil.readFileToString(sourceFile));
            FileUtil.writeStringToFile(compress, targetFile, false, false);
            // 目标文件压缩后的大小
            long targetLength = targetFile.length();
            // 打印压缩信息
            getLog().info(targetFile.getName() + FileUtil.compressorInfo(sourceLength, targetLength));
        } catch (Exception e) {
            e.printStackTrace();
            getLog().info(sourceFile.getAbsolutePath() + " compress failed");
        }
    }

    public void compressXML(File sourceFile, File targetFile) {
        try {
            // 源文件大小
            long sourceLength = sourceFile.length();
            // 从源文件向目标复制文件
            FileUtils.copyFile(sourceFile, targetFile);
            // 对目标文件进行压缩
            XmlCompressor compressor = new XmlCompressor();
            String compress = compressor.compress(FileUtil.readFileToString(sourceFile));
            FileUtil.writeStringToFile(compress, targetFile, false, false);
            // 目标文件压缩后的大小
            long targetLength = targetFile.length();
            // 打印压缩信息
            getLog().info(targetFile.getName() + FileUtil.compressorInfo(sourceLength, targetLength));
        } catch (Exception e) {
            e.printStackTrace();
            getLog().info(sourceFile.getAbsolutePath() + " compress failed");
        }
    }
}
